require "papi.so"

module PAPI
  module Util
    module_function
    def to_ruby_class_name(name)
      name.split("_").collect do |x|
        "#{x[0,1].upcase}#{x[1..-1].downcase}"
      end.join("")
    end
  end

  class Error < StandardError
    class << self
      def corresponding_error(status)
        constants.each do |name|
          klass = const_get(name)
          return klass if klass.status == status
        end
        raise "Unknown error status: #{status}"
      end
    end


    Status.constants.each do |name|
      status = Status.const_get(name)
      module_eval(<<-EOS, __FILE__, __LINE__ + 1)
        class #{Util.to_ruby_class_name(name)} < Error
          class << self
            def status
              #{status}
            end

            def string
              Status.string(status)
            end
          end

          def status
            self.class.status
          end

          def string
            self.class.string
          end
        end
EOS
    end
  end

  class AttributeList
    def [](name)
      attr = find(name)
      if attr
        attr.values
      else
        nil
      end
    end
  end

  class Job
    alias job_ticket ticket
  end

  class Metadata
    constants.each do |name|
      value = const_get(name)
      module_eval(<<-EOS, __FILE__, __LINE__ + 1)
        class #{Util.to_ruby_class_name(name)} < Metadata
          class << self
            def value
              #{value}
            end
          end

          attr_reader :value
          def initialize
            @value = value
          end

          def to_s
            "#<#{self.class.name}:#{name}>"
          end
        end
EOS
    end
  end
end
